% The shape parameter
ep=0.5;

% Use Halton nodes in the unit square
N = 200; d=2;
xk = halton(N,d);

% Evaluation points
Ne = 40; % Per dimension
x = linspace(0,1,Ne);
[xx,yy]=meshgrid(x);
xe = [xx(:) yy(:)];

% Plot nodes
figure(1),clf
plot(xk(:,1),xk(:,2),'bo',xx,yy,'k-',xx',yy','k-')
axis equal
title('Node points: o, Evaluation points: grid')

tic;
[A,Psi] = RBF_QR_diffmat_2D('1',xe,xk,ep);
[Ax] = RBF_QR_diffmat_2D('x',xe,Psi);
[Ay] = RBF_QR_diffmat_2D('y',xe,Psi);
[Axx] = RBF_QR_diffmat_2D('xx',xe,Psi);
[Axy] = RBF_QR_diffmat_2D('xy',xe,Psi);
[Ayy] = RBF_QR_diffmat_2D('yy',xe,Psi);
[L] = RBF_QR_diffmat_2D('L',xe,Psi);
separate=toc
op = {'1','x','y','xx','xy','yy','L'};

tic;
[Acell,Psi] = RBF_QR_diffmat_2D(op,xe,xk,ep); 
                                              
allinone=toc
%
% Apply to a function and compare with exact solution
%
uk = f('0',xk); 
diff = A*uk-f('0',xe);
diffx = Ax*uk-f('x',xe);
diffy = Ay*uk-f('y',xe);
diffxx = Axx*uk-f('xx',xe);
diffxy = Axy*uk-f('xy',xe);
diffyy = Ayy*uk-f('yy',xe);
diffL = L*uk-f('L',xe);

%
% Mesh the solution
%
figure(2)
mesh(xx,yy,reshape(f('0',xe),size(xx)))
%
% Mesh the interpolation error, the x-derivative error, and the Laplacian error
%
figure(3)
mesh(xx,yy,reshape(diff,size(xx)))
figure(4)
mesh(xx,yy,reshape(diffx,size(xx)))
figure(5)
mesh(xx,yy,reshape(diffL,size(xx)))
