%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) 2015 Elisabeth Larsson
%
% DEMO3: Interpolate a function in 2-D on a star shaped domain using
% Gaussian RBFs and both the direct approach and RBF-QR. 
%
% Compare the convergence as a function of N for the two methods.   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% The function to interpolate and its first derivatives
%
fun = @(x1,x2) sin(x1.^2+2*x2.^2) - sin(2*x1.^2+(x2-0.5).^2);
dfun1 = @(x1,x2) 2*x1.*cos(x1.^2 + 2*x2.^2) ...
        - 4*x1.*cos((x2 - 1/2).^2 + 2*x1.^2);
dfun2 = @(x1,x2) 4*x2.*cos(x1.^2 + 2*x2.^2) ...
        - cos((x2 - 1/2).^2 + 2*x1.^2).*(2*x2 - 1);
%
% A star shaped domain has a boundary that can be expressed as r(theta)
%
% This is a flower domain
rb = @(theta) 1+0.8*(cos(4*theta));
R = 1.8; % The maximum radius
%
% This one we call a starfish
% rb = @(theta) 1+0.1*(sin(6*theta)+sin(3*theta));
% R = 1.2; %Actually less, this is an upper bound

%
% Use Halton nodes in the interior. Start from nodes in a square.
%
ndims=2;
N0 = 100;
x0 = R*(-1 + 2*halton(N0,ndims));
%
% Estimate the fraction of the square covered by the domain.
%
[th,r]=cart2pol(x0(:,1),x0(:,2));
in = find(r<rb(th));
Aomega = length(in)/N0;
%
% Use estimated arclength to distribute boundary nodes
%
theta0=0:2*pi/N0:2*pi;
r = rb(theta0);

x = r.*cos(theta0);
y = r.*sin(theta0);
%
% Measure the Euclidian length of each theta interval
%
l = sqrt(diff(x).^2 + diff(y).^2);
L = [0 cumsum(l)];
%
% L(theta) is a monotone function that can be inverted to give theta(L).
% That is, we can find theta values that give a uniform distribution of
% arclength between nodes.
% 
lg = linspace(0,L(end),Nb);
% We get the function theta(L)
thb = interp1(L,theta,lg)
%
% Select approximate node numbers for the interpolation
% 
Nvec = [25 50 100 200 400 800];

% We need the function of N effect where we get into trouble.

% First choose ep through a plot.

% Then run for a series of N and see how that goes.

% The shape parameter
ep=0.5;

% Use Halton nodes in the unit square
N = 200; d=2;
xk = halton(N,d);

% Evaluation points
Ne = 40; % Per dimension
x = linspace(0,1,Ne);
[xx,yy]=meshgrid(x);
xe = [xx(:) yy(:)];

% Plot nodes
figure(1),clf
plot(xk(:,1),xk(:,2),'r*',xx,yy,'k-',xx',yy','k-')
axis equal
title('Node points: o, Evaluation points: grid')

tic;
[A,Psi] = RBF_QR_diffmat_2D('1',xe,xk,ep);
[Ax] = RBF_QR_diffmat_2D('x',xe,Psi);
[Ay] = RBF_QR_diffmat_2D('y',xe,Psi);
[Axx] = RBF_QR_diffmat_2D('xx',xe,Psi);
[Axy] = RBF_QR_diffmat_2D('xy',xe,Psi);
[Ayy] = RBF_QR_diffmat_2D('yy',xe,Psi);
[L] = RBF_QR_diffmat_2D('L',xe,Psi);
separate=toc
op = {'1','x','y','xx','xy','yy','L'};

tic;
[Acell,Psi] = RBF_QR_diffmat_2D(op,xe,xk,ep); 
                                              
allinone=toc
%
% Apply to a function and compare with exact solution
%
uk = f('0',xk); 
diff = A*uk-f('0',xe);
diffx = Ax*uk-f('x',xe);
diffy = Ay*uk-f('y',xe);
diffxx = Axx*uk-f('xx',xe);
diffxy = Axy*uk-f('xy',xe);
diffyy = Ayy*uk-f('yy',xe);
diffL = L*uk-f('L',xe);

%
% Mesh the solution
%
figure(2)
mesh(xx,yy,reshape(f('0',xe),size(xx)))
%
% Mesh the interpolation error, the x-derivative error, and the Laplacian error
%
figure(3)
mesh(xx,yy,reshape(diff,size(xx)))
figure(4)
mesh(xx,yy,reshape(diffx,size(xx)))
figure(5)
mesh(xx,yy,reshape(diffL,size(xx)))
